function spatiogram = compute_meanshift_spatiogram(img, state, aspect_ratio, nbins)

cen_r = state(1);
cen_c = state(2);

% ellipse axes
axis_horiz = state(3);
axis_vert = round(state(3)*aspect_ratio);

[perim, mask] = generate_ellipse(size(img), state, aspect_ratio);

%figure(5), imagesc(mask)

%ellipse_perimeter = ellipse(state, aspect_ratio);
%ellipse_mask = create_ellipse_binary_mask(img,ellipse_perimeter);
col1 = img(:,:,1);
col2 = img(:,:,2);
col3 = img(:,:,3);

binwidth1 = 256/nbins(1);
binwidth2 = 256/nbins(2);
binwidth3 = 256/nbins(3);
spatiogram.histogram = zeros(nbins(1),nbins(2),nbins(3));
n = zeros(nbins(1),nbins(2),nbins(3));
sumc = zeros(nbins(1),nbins(2),nbins(3));
sumr = zeros(nbins(1),nbins(2),nbins(3));
sumc2 = zeros(nbins(1),nbins(2),nbins(3));
sumr2 = zeros(nbins(1),nbins(2),nbins(3));
for r=1:size(img,1)
    for c=1:size(img,2)
        if (mask(r, c) == 1)
            indx1 = floor( col1(r,c) / binwidth1 ) + 1;
            indx2 = floor( col2(r,c) / binwidth2 ) + 1;
            indx3 = floor( col3(r,c) / binwidth3 ) + 1;
            cc = c-cen_c;
            rr = r-cen_r;
%            [cc rr]
            amt = kernel_func([rr / axis_vert, cc / axis_horiz]);
            spatiogram.histogram(indx1, indx2, indx3) = spatiogram.histogram(indx1, indx2, indx3) + amt;
            n(indx1, indx2, indx3) = n(indx1, indx2, indx3) + 1;
            sumc(indx1, indx2, indx3) = sumc(indx1, indx2, indx3) + cc;
            sumr(indx1, indx2, indx3) = sumr(indx1, indx2, indx3) + rr;
            sumc2(indx1, indx2, indx3) = sumc2(indx1, indx2, indx3) + cc*cc;
            sumr2(indx1, indx2, indx3) = sumr2(indx1, indx2, indx3) + rr*rr;
        end
    end
end

% compute means and variances
% special cases:
%     0.  If there are no pixels in a bin, the mean and var are zero
%     1.  If there is exactly one pixel in a bin, the var is set (arbitrarily) to 1 
n = max(n, 1);  % prevents divide by zero
spatiogram.mean_c = sumc ./ n;
spatiogram.mean_r = sumr ./ n;
spatiogram.var_c = sumc2 ./ n - spatiogram.mean_c.^2;
spatiogram.var_r = sumr2 ./ n - spatiogram.mean_r.^2;
spatiogram.var_c = max(spatiogram.var_c, 1);
spatiogram.var_r = max(spatiogram.var_r, 1);

% normalize histogram
spatiogram.histogram = spatiogram.histogram / sum(sum(sum(spatiogram.histogram)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% kernel function; input is (r,c) normalized coordinates of point
% This code uses the Epanechnikov profile
function out = kernel_func(point)
p = norm(point);
%aaa = [point p]
if p>1, 
    out = 0;
    if p>1.1, error('assert');  end
else
    d = 2;
    cd = pi;
    out = (1-p)*(d+2) / (2*cd);
end